package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.converter.OfferConverter;
import com.nisum.offertransactionservice.converter.PEResponseToOfferTransactionResConverter;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.nio.charset.Charset;
import java.time.ZonedDateTime;
import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@Service
@PropertySource("classpath:application.properties")
public class OfferCallingPEService {
    @Autowired
    private OfferConverter offerConverter;

    @Autowired
    private OfferLookupRepo offerLookupRepo;

    @Autowired
    private PEResponseToOfferTransactionResConverter peResponseToOfferTransactionResConverter;

    @Autowired
    private WebClient webClient;

    @Value("${promotion.engine.calculate.discount.url}")
    private String promotionEngineUrl;

    public OfferTransactionResponse getDiscountedItemList(OfferTransactionRequest offerTransactionRequest) {
        List<OfferLookup> eligibleOffer = newArrayList();
        offerLookupRepo.findAll().forEach(eligibleOffer::add);
        OfferTransactionResponse offerTransactionResponse;
        PERequest peRequest = offerConverter.apply(offerTransactionRequest);
        peRequest.setEligibleOffers(eligibleOffer);
        Flux<PEResponse> peResponseFlux = webClient.post().uri("http://localhost:8081" + promotionEngineUrl).accept(MediaType.APPLICATION_JSON)
                .contentType(MediaType.APPLICATION_JSON).body(Mono.just(peRequest), PERequest.class).retrieve().bodyToFlux(PEResponse.class);
        offerTransactionResponse = peResponseToOfferTransactionResConverter.apply(peResponseFlux.toStream().findFirst().get());
        return offerTransactionResponse;
    }
}
